using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public enum ObstacleType
{
	E_Undefined,
	E_Alien,
	E_DarkRoom,
	E_Dog,
	E_Door_breakable,
	E_Door_with_key,
	E_Fire,
	E_ParasitedHuman,
	E_Zombie
}

public class LevelManager : MonoBehaviour {

	[SerializeField]
	private GameObject m_StudentGroupPrefab = null;

	[SerializeField]
	private GameObject m_WonMessagePrefab = null;

	[SerializeField]
	private GameObject m_LostMessagePrefab = null;

	[SerializeField]
	private Transform m_EndGameMessageLocator = null;

	private GameObject m_EndGameMessage = null;

	// Array of rooms
	//[SerializeField]
	private Room [] m_rooms = null;

	// Array of room connections
	//[SerializeField]
	private RoomConnection [] m_roomConnections = null;
	
	[SerializeField]
	private GameObject[] m_roomObstacles;

	[SerializeField]
	private GameObject[] m_connectionObstacles;
	
	[SerializeField]
	private GameObject[] m_useablesPrefab;

	[SerializeField]
	private float m_EndGameWaitTime = 0.0f;

	private float m_EndGameWaitTimeLeft = 0.0f;

	CameraManager m_cameraManager;

	Room m_startRoom = null;

	[SerializeField]
	private GameObject m_startPrefab;
	private GameObject m_startPrefabObject;

	// Use this for initialization
	void Start()
	{

		// Get the rooms
		m_rooms = this.GetComponentsInChildren<Room>();
		System.Diagnostics.Debug.Assert(m_rooms.Length > 0);

		// Get the room connections
		m_roomConnections = this.GetComponentsInChildren<RoomConnection>();
		System.Diagnostics.Debug.Assert(m_roomConnections.Length > 0);

		Random.seed = System.Environment.TickCount;
		//Random.seed = 42;

		SpawnUseables();

		m_cameraManager = this.GetComponent<CameraManager>();

		RoomSelector roomSelector = this.GetComponent<RoomSelector>();
		m_startRoom = roomSelector.StartSelection(this, m_rooms, OnSequenceSelectionDone);

		if (m_startPrefab != null)
		{
			m_startPrefabObject = Instantiate(m_startPrefab, m_startRoom.transform.position, m_startRoom.transform.rotation) as GameObject;
		}

		enabled = false;
	}

	void SpawnUseables()
	{
		List<UseableSpawner> useableSpawners = new List<UseableSpawner>();
		foreach(Room r in m_rooms)
		{
			UseableSpawner spawner = r.GetComponent<UseableSpawner>();
			if (spawner != null)
			{
				useableSpawners.Add(spawner);
			}
		}
		
		foreach(GameObject useablePrefab in m_useablesPrefab)
		{
			int spawnerCount = useableSpawners.Count;
			if (spawnerCount > 0)
			{
				int spawnerIndex = Random.Range(0, spawnerCount - 1);
				Transform spawnerTransform = useableSpawners[spawnerIndex].transform;
				GameObject useableObject = Instantiate(useablePrefab, spawnerTransform.position, spawnerTransform.rotation) as GameObject;
				useableObject.transform.parent = spawnerTransform;
				
				useableSpawners.RemoveAt(spawnerIndex);
			}
		}
	}

	void SpawnObstacles(Room startRoom)
	{
		List<ObstacleSpawner> obstacleSpawners = new List<ObstacleSpawner>();
		foreach(RoomConnection connection in m_roomConnections)
		{
			ObstacleSpawner spawner = connection.GetComponent<ObstacleSpawner>();
			if (spawner != null)
			{
				obstacleSpawners.Add(spawner);
			}
		}

		foreach(GameObject obstaclePrefab in m_connectionObstacles)
		{
			int spawnerCount = obstacleSpawners.Count;
			if (spawnerCount > 0)
			{
				int spawnerIndex = Random.Range(0, spawnerCount - 1);
				Transform spawnTransform = obstacleSpawners[spawnerIndex].transform;
				GameObject obstacleObject = Instantiate(obstaclePrefab, spawnTransform.position, spawnTransform.rotation) as GameObject;
				obstacleObject.transform.parent = spawnTransform;
				
				obstacleSpawners.RemoveAt(spawnerIndex);
			}
		}

		obstacleSpawners.Clear();
		foreach(Room room in m_rooms)
		{
			ObstacleSpawner spawner = room.GetComponent<ObstacleSpawner>();
			if (spawner != null && room != startRoom)
			{
				obstacleSpawners.Add(spawner);
			}
		}
		
		foreach(GameObject obstaclePrefab in m_roomObstacles)
		{
			int spawnerCount = obstacleSpawners.Count;
			if (spawnerCount > 0)
			{
				int spawnerIndex = Random.Range(0, spawnerCount - 1);
				Transform spawnTransform = obstacleSpawners[spawnerIndex].transform;
				GameObject obstacleObject = Instantiate(obstaclePrefab, spawnTransform.position, spawnTransform.rotation) as GameObject;
				obstacleObject.transform.parent = spawnTransform;
				
				obstacleSpawners.RemoveAt(spawnerIndex);
			}
		}
	}

	public void OnSequenceSelectionDone(List<Room> roomSequence)
	{
		if (m_startPrefabObject != null)
		{
			Destroy(m_startPrefabObject);
			m_startPrefabObject = null;
		}

		System.Diagnostics.Debug.Assert(roomSequence.Count > 0);
		SpawnObstacles(roomSequence[0]);

		SequenceUpdater sequenceUpdater = this.GetComponent<SequenceUpdater>();
		sequenceUpdater.StartSequence(this, roomSequence, m_StudentGroupPrefab);
	}
	
	// Update is called once per frame
	void Update()
	{
		float deltaTime = Time.deltaTime;

		if (m_EndGameWaitTimeLeft > deltaTime)
		{
			m_EndGameWaitTimeLeft -= deltaTime;
		}
		else
		{
			Destroy(m_EndGameMessage);
			m_EndGameMessage = null;

			LoadNextLevel();

			m_EndGameWaitTimeLeft = 0.0f;
			enabled = false;
		}
	}

	public void EndGame(bool playerWon)
	{
		if (playerWon)
		{
			m_EndGameMessage = Instantiate(m_WonMessagePrefab, m_EndGameMessageLocator.position, m_EndGameMessageLocator.rotation) as GameObject;
		}
		else
		{
			m_EndGameMessage = Instantiate(m_LostMessagePrefab, m_EndGameMessageLocator.position, m_EndGameMessageLocator.rotation) as GameObject;
		}

		m_EndGameWaitTimeLeft = m_EndGameWaitTime;

		enabled = true;
	}

	// Get the connection between two rooms
	public RoomConnection GetConnectionBetweenRooms(Room r1, Room r2)
	{
		RoomConnection foundConnection = null;

		foreach (RoomConnection c in m_roomConnections)
		{
			if ( c.IsConnectingRooms(r1, r2) )
			{
				foundConnection = c;
				break;
			}
		}

		return foundConnection;
	}

	public CameraManager GetCameraManager()
	{
		return m_cameraManager;
	}

	public int GetCurrentLevelIndex() { return Application.loadedLevel; }
	public int GetLevelCount() { return Application.levelCount; }

	public void LoadLevel(int level)
	{
		int levelCount = GetLevelCount();
		int levelToLoad = level % levelCount;
		
		Debug.Log( string.Format("Loading level {0} ...", levelToLoad) );
		
		//m_LoadingLevel = true;
		
		Application.LoadLevel(levelToLoad);
	}

	public void LoadNextLevel()
	{
		int currentLevelIndex = GetCurrentLevelIndex();
		int levelCount = GetLevelCount();
		int nextLevelIndex = (currentLevelIndex + 1) % levelCount;
		
		LoadLevel(nextLevelIndex);
	}
}
