﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class SequenceUpdater : MonoBehaviour {

	[SerializeField]
	private float m_waitTimeBetweenSteps = 0.0f;

	private float m_waitTimeLeft = 0.0f;

	private LevelManager m_levelManager = null;

	private GameObject m_StudentGroup = null;
	
	private List<Room> m_roomSequence = null;
	
	[SerializeField]
	private int m_currentSequenceIndex = -1;


	// Use this for initialization
	void Start()
	{
		PauseSequence();
	}

	void PauseSequence()
	{
		enabled = false;
	}

	void UnpauseSequence()
	{
		enabled = true;
	}
	
	// Update is called once per frame
	void Update()
	{
		float deltaTime = Time.deltaTime;
		if (m_waitTimeLeft < deltaTime)
		{
			m_waitTimeLeft = 0.0f;

			Debug.Log("Sequence step " + m_currentSequenceIndex + " ...");

			Room currentRoom = m_roomSequence[m_currentSequenceIndex];
			LootRoom(currentRoom);

			int nextSequenceIndex = m_currentSequenceIndex + 1;
			int lastSequenceIndex = m_roomSequence.Count;

			if (nextSequenceIndex != lastSequenceIndex)
			{
				Room nextRoom = m_roomSequence[nextSequenceIndex];
				RoomConnection toNext = m_levelManager.GetConnectionBetweenRooms(currentRoom, nextRoom);

				UpdateSequence(currentRoom, nextRoom, toNext);

				m_currentSequenceIndex = nextSequenceIndex;
			}
			else
			{
				//Final obstacle is supposed to be in final room
				GroupPathObstacle obstacle = currentRoom.GetComponentInChildren<GroupPathObstacle>();
				System.Diagnostics.Debug.Assert(obstacle != null);

				GroupInventory inventory = m_StudentGroup.GetComponent<GroupInventory>();
				bool hasFinalKey = inventory.hasObjectAgainstObstacle(obstacle);

				EndSequence(hasFinalKey);
			}
		}
		else
		{
			m_waitTimeLeft -= deltaTime;
		}
	}

	public void StartSequence(LevelManager levelManager, List<Room> roomSequence, GameObject studentGroupPrefab)
	{
		System.Diagnostics.Debug.Assert(levelManager != null);
		System.Diagnostics.Debug.Assert(roomSequence != null && roomSequence.Count > 0);
		System.Diagnostics.Debug.Assert(studentGroupPrefab != null);
		
		System.Diagnostics.Debug.Assert(m_levelManager == null);
		System.Diagnostics.Debug.Assert(m_StudentGroup == null);
		System.Diagnostics.Debug.Assert(m_roomSequence == null);
		System.Diagnostics.Debug.Assert(m_currentSequenceIndex == -1);

		m_levelManager = levelManager;
		m_roomSequence = roomSequence;
		m_currentSequenceIndex = 0;
		
		Room startRoom = m_roomSequence[m_currentSequenceIndex];
		Transform groupTransform = GetGroupTransformInRoom(startRoom);
		
		m_StudentGroup = Instantiate(studentGroupPrefab, groupTransform.position, groupTransform.rotation) as GameObject;
		System.Diagnostics.Debug.Assert(m_StudentGroup != null);

		m_waitTimeLeft = m_waitTimeBetweenSteps;
		
		UnpauseSequence();
	}

	void UpdateSequence(Room currentRoom, Room nextRoom, RoomConnection toNext)
	{
		GroupHealth health = m_StudentGroup.GetComponent<GroupHealth>();
		GroupInventory inventory = m_StudentGroup.GetComponent<GroupInventory>();

		GroupPathObstacle obstacle = currentRoom.GetComponentInChildren<GroupPathObstacle>();
		if (obstacle != null)
		{
			if (inventory.hasObjectAgainstObstacle(obstacle))
			{
				Destroy(obstacle.gameObject);
				obstacle = null;
			}
			else
			{
				health.TakeDamageFromObstacle(obstacle);
			}
		}
		
		obstacle = toNext.GetComponentInChildren<GroupPathObstacle>();
		if (obstacle != null)
		{
			if (inventory.hasObjectAgainstObstacle(obstacle))
			{
				Destroy(obstacle.gameObject);
				obstacle = null;
			}
			else
			{
				health.TakeDamageFromObstacle(obstacle);
			}
		}
		
		if (health.IsStillAlive())
		{
			Transform nextRoomTransform = GetGroupTransformInRoom(nextRoom);
			MoveGroup(nextRoomTransform, toNext.transform);

			m_waitTimeLeft = m_waitTimeBetweenSteps;
		}
		else
		{
			EndSequence(false);
		}
	}

	public void EndSequence(bool playerWon)
	{
		Debug.Log("End of sequence...");

		PauseSequence();

		m_levelManager.EndGame(playerWon);
	}

	void LootRoom(Room r)
	{
		ObjectUseable useable = r.GetComponentInChildren<ObjectUseable>();
		if (useable != null)
		{
			GroupInventory inventory = m_StudentGroup.GetComponent<GroupInventory>();
			inventory.PickupObject(useable.gameObject);

			useable.transform.parent = inventory.transform;
		}
	}

	Transform GetGroupTransformInRoom(Room r)
	{
		GroupPathPoint pathPoint = r.GetComponentInChildren<GroupPathPoint> ();
		System.Diagnostics.Debug.Assert(pathPoint != null);
		
		return r.transform;
	}
	
	void MoveGroup(Transform destination, Transform intermediate)
	{
		PathFollower groupMoveUpdater = m_StudentGroup.GetComponent<PathFollower>();
		System.Diagnostics.Debug.Assert(m_StudentGroup == null);

		List<Transform> path = new List<Transform>();
		path.Add(intermediate);
		path.Add(destination);

		groupMoveUpdater.StartMove(path, OnMoveGroupFinished, m_levelManager.GetCameraManager());

		PauseSequence();
	}
	
	public void OnMoveGroupFinished()
	{
		UnpauseSequence();
	}
}